@@ -47,6 +47,9 @@ module XMonad.Layout.LayoutCombinators
       -- $jtl
     , (|||)
     , JumpToLayout(..)
+
+      -- * Types
+    , NewSelect
     ) where
 
 import Data.Maybe ( isJust, isNothing )
@@ -76,6 +79,7 @@ import XMonad.Layout.DragPane
 -- > import XMonad hiding ( (|||) )
 -- > import XMonad.Layout.LayoutCombinators
 --
+-- If you import XMonad.Layout, you will need to hide it from there as well.
 -- Then bind some keys to a 'JumpToLayout' message:
 --
 -- >   , ((modm .|. controlMask, xK_f), sendMessage $ JumpToLayout "Full")  -- jump directly to the Full layout
@@ -179,9 +183,16 @@ infixr 5 |||
 -- The standard xmonad core exports a layout combinator @|||@ which
 -- represents layout choice.  This is a reimplementation which also
 -- provides the capability to support 'JumpToLayout' messages.  To use
--- it, be sure to hide the import of @|||@ from the xmonad core:
+-- it, be sure to hide the import of @|||@ from the xmonad core; if either of
+-- these two lines appear in your configuration:
+--
+-- > import XMonad
+-- > import XMonad.Layout
+--
+-- replace them with these instead, respectively:
 --
 -- > import XMonad hiding ( (|||) )
+-- > import XMonad.Layout hiding ( (|||) )
 --
 -- The argument given to a 'JumpToLayout' message should be the
 -- @description@ of the layout to be selected.  If you use
@@ -213,7 +224,7 @@ infixr 5 |||
 
 data NewSelect l1 l2 a = NewSelect Bool (l1 a) (l2 a) deriving ( Read, Show )
 
--- | 
+-- |
 data JumpToLayout = JumpToLayout String -- ^ A message to jump to a particular layout
                                         -- , specified by its description string..
                   | NextLayoutNoWrap
