@@ -1,4 +1,4 @@
-{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses, DeriveDataTypeable, PatternGuards #-}
+{-# LANGUAGE CPP, FlexibleInstances, MultiParamTypeClasses, DeriveDataTypeable, PatternGuards #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.LimitWindows
@@ -23,13 +23,20 @@ module XMonad.Layout.LimitWindows (
     limitWindows,limitSlice,limitSelect,
 
     -- * Change the number of windows
-    increaseLimit,decreaseLimit,setLimit
+    increaseLimit,decreaseLimit,setLimit,
+
+#ifdef TESTING
+    -- * For tests
+    select,update,Selection(..),updateAndSelect,
+#endif
+
+    -- * Types
+    LimitWindows, Selection,
     ) where
 
 import XMonad.Layout.LayoutModifier
 import XMonad
 import qualified XMonad.StackSet as W
-import XMonad.Layout (IncMasterN (..))
 import Control.Monad((<=<),guard)
 import Control.Applicative((<$>))
 import Data.Maybe(fromJust)
@@ -139,7 +146,7 @@ select s stk
         downs = W.down stk
         ups = reverse $ W.up stk
         lups = length ups
-    
+
 updateStart :: Selection l -> W.Stack a -> Int
 updateStart s stk
     | lups < nMaster s  -- the focussed window is in the master pane
