@@ -1,5 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-} -- For deriving Data/Typeable
-{-# LANGUAGE FlexibleInstances, GeneralizedNewtypeDeriving, MultiParamTypeClasses #-}
+{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, GeneralizedNewtypeDeriving, MultiParamTypeClasses #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -34,8 +33,8 @@ import Data.List ((\\))
 --
 -- Then edit your @layoutHook@ by adding the ResizableTile layout:
 --
--- > myLayouts =  ResizableTall 1 (3/100) (1/2) [] ||| etc..
--- > main = xmonad defaultConfig { layoutHook = myLayouts }
+-- > myLayout =  ResizableTall 1 (3/100) (1/2) [] ||| etc..
+-- > main = xmonad defaultConfig { layoutHook = myLayout }
 --
 -- For more detailed instructions on editing the layoutHook see:
 --
@@ -43,8 +42,8 @@ import Data.List ((\\))
 --
 -- You may also want to add the following key bindings:
 --
--- > , ((modMask x,               xK_a), sendMessage MirrorShrink)
--- > , ((modMask x,               xK_z), sendMessage MirrorExpand)
+-- > , ((modm,               xK_a), sendMessage MirrorShrink)
+-- > , ((modm,               xK_z), sendMessage MirrorExpand)
 --
 -- For detailed instruction on editing the key binding see:
 --
@@ -53,7 +52,20 @@ import Data.List ((\\))
 data MirrorResize = MirrorShrink | MirrorExpand deriving Typeable
 instance Message MirrorResize
 
-data ResizableTall a = ResizableTall Int Rational Rational [Rational] deriving (Show, Read)
+data ResizableTall a = ResizableTall
+    { _nmaster :: Int       -- ^ number of master windows
+    , _delta  :: Rational   -- ^ change when resizing by 'Shrink', 'Expand',
+                            -- 'MirrorShrink', 'MirrorExpand'
+    , _frac   :: Rational   -- ^ width of master
+    , _slaves :: [Rational] -- ^ fraction to multiply the window
+                            -- height that would be given when divided equally.
+                            --
+                            -- slave windows are assigned their modified
+                            -- heights in order, from top to bottom
+                            --
+                            -- unspecified values are replaced by 1
+    } deriving (Show, Read)
+
 instance LayoutClass ResizableTall a where
     doLayout (ResizableTall nmaster _ frac mfrac) r =
         return . (\x->(x,Nothing)) .
@@ -95,7 +107,7 @@ splitVertically [] _ r = [r]
 splitVertically _ n r | n < 2 = [r]
 splitVertically (f:fx) n (Rectangle sx sy sw sh) = Rectangle sx sy sw smallh :
     splitVertically fx (n-1) (Rectangle sx (sy+fromIntegral smallh) sw (sh-smallh))
-  where smallh = floor $ fromIntegral (sh `div` fromIntegral n) * f --hmm, this is a fold or map.
+  where smallh = min sh (floor $ fromIntegral (sh `div` fromIntegral n) * f) --hmm, this is a fold or map.
 
 splitHorizontallyBy :: RealFrac r => r -> Rectangle -> (Rectangle, Rectangle)
 splitHorizontallyBy f (Rectangle sx sy sw sh) =
