@@ -22,7 +22,8 @@ module XMonad.Prompt.AppendFile (
                                  -- * Usage
                                  -- $usage
 
-                                 appendFilePrompt
+                                 appendFilePrompt,
+                                 AppendFile,
                                 ) where
 
 import XMonad.Core
@@ -43,6 +44,17 @@ import Control.Exception.Extensible (bracket)
 --
 -- >  , ((modm .|. controlMask, xK_n), appendFilePrompt defaultXPConfig "/home/me/NOTES")
 --
+-- Additional notes can be added via regular Haskell or XMonad functions; for
+-- example, to preface notes with the time they were made, one could write a
+-- binding like
+--
+-- > ,  ((modm .|. controlMask, xK_n), do
+-- >            spawn ("date>>"++"/home/me/NOTES")
+-- >            appendFilePrompt defaultXPConfig "/home/me/NOTES"
+-- >        )
+--
+-- (Put the spawn on the line after the prompt to append the time instead.)
+--
 -- For detailed instructions on editing your key bindings, see
 -- "XMonad.Doc.Extending#Editing_key_bindings".
 
