@@ -4,7 +4,7 @@
 -- Copyright   :  (C) 2007 Andrea Rossato, David Roundy
 -- License     :  BSD3
 --
--- Maintainer  :  
+-- Maintainer  :
 -- Stability   :  unstable
 -- Portability :  unportable
 --
@@ -15,7 +15,8 @@
 module XMonad.Prompt.Directory (
                              -- * Usage
                              -- $usage
-                             directoryPrompt
+                             directoryPrompt,
+                             Dir,
                               ) where
 
 import XMonad
