@@ -4,7 +4,7 @@
 -- Copyright   :  (C) 2007 Andrea Rossato, David Roundy
 -- License     :  BSD3
 --
--- Maintainer  :  
+-- Maintainer  :
 -- Stability   :  unstable
 -- Portability :  unportable
 --
@@ -21,6 +21,7 @@ module XMonad.Prompt.Layout (
 import Data.List ( sort, nub )
 import XMonad hiding ( workspaces )
 import XMonad.Prompt
+import XMonad.Prompt.Workspace ( Wor(..) )
 import XMonad.StackSet ( workspaces, layout )
 import XMonad.Layout.LayoutCombinators ( JumpToLayout(..) )
 
@@ -43,11 +44,6 @@ import XMonad.Layout.LayoutCombinators ( JumpToLayout(..) )
 -- more a proof-of-principle than something you can actually use
 -- productively.
 
-data Wor = Wor String
-
-instance XPrompt Wor where
-    showXPrompt (Wor x) = x
-
 layoutPrompt :: XPConfig -> X ()
 layoutPrompt c = do ls <- gets (map (description . layout) . workspaces . windowset)
                     mkXPrompt (Wor "") c (mkComplFunFromList' $ sort $ nub ls) (sendMessage . JumpToLayout)
