@@ -19,15 +19,13 @@ module XMonad.Prompt.Ssh
       Ssh,
     ) where
 
-import Prelude hiding (catch)
-
 import XMonad
 import XMonad.Util.Run
 import XMonad.Prompt
 
 import System.Directory
 import System.Environment
-import Control.Exception
+import Control.Exception as E
 
 import Control.Monad
 import Data.Maybe
@@ -78,7 +76,7 @@ sshComplListLocal = do
 
 sshComplListGlobal :: IO [String]
 sshComplListGlobal = do
-  env <- getEnv "SSH_KNOWN_HOSTS" `catch` econst "/nonexistent"
+  env <- getEnv "SSH_KNOWN_HOSTS" `E.catch` econst "/nonexistent"
   fs <- mapM fileExists [ env
                         , "/usr/local/etc/ssh/ssh_known_hosts"
                         , "/usr/local/etc/ssh_known_hosts"
