@@ -15,12 +15,12 @@ module XMonad.Prompt.Theme
     ( -- * Usage
       -- $usage
       themePrompt,
+      ThemePrompt,
     ) where
 
 import Control.Arrow ( (&&&) )
 import qualified Data.Map as M
 import Data.Maybe ( fromMaybe )
-import Data.List
 import XMonad
 import XMonad.Prompt
 import XMonad.Layout.Decoration
