@@ -27,7 +27,9 @@ module XMonad.Util.EZConfig (
                              -- * Emacs-style keybinding specifications
 
                              mkKeymap, checkKeymap,
-                             mkNamedKeymap
+                             mkNamedKeymap,
+
+                             parseKey -- used by XMonad.Util.Paste
                             ) where
 
 import XMonad
@@ -103,7 +105,7 @@ additionalKeysP conf keyList =
 -- >                 `removeKeys` [(mod1Mask .|. shiftMask, n) | n <- [xK_1 .. xK_9]]
 removeKeys :: XConfig a -> [(ButtonMask, KeySym)] -> XConfig a
 removeKeys conf keyList =
-    conf { keys = \cnf -> keys conf cnf `M.difference` M.fromList (zip keyList $ return ()) }
+    conf { keys = \cnf -> keys conf cnf `M.difference` M.fromList (zip keyList $ repeat ()) }
 
 -- | Like 'removeKeys', except using short @String@ key descriptors
 --   like @\"M-m\"@ instead of @(modMask, xK_m)@, as described in the
@@ -125,7 +127,7 @@ additionalMouseBindings conf mouseBindingsList =
 removeMouseBindings :: XConfig a -> [(ButtonMask, Button)] -> XConfig a
 removeMouseBindings conf mouseBindingList =
     conf { mouseBindings = \cnf -> mouseBindings conf cnf `M.difference`
-                                   M.fromList (zip mouseBindingList $ return ()) }
+                                   M.fromList (zip mouseBindingList $ repeat ()) }
 
 
 --------------------------------------------------------------
@@ -343,6 +345,7 @@ removeMouseBindings conf mouseBindingList =
 -- > <XF86MailForward>
 -- > <XF86Pictures>
 -- > <XF86Music>
+-- > <XF86TouchpadToggle>
 -- > <XF86_Switch_VT_1>-<XF86_Switch_VT_12>
 -- > <XF86_Ungrab>
 -- > <XF86_ClearGrab>
@@ -410,9 +413,9 @@ parseModifier :: XConfig l -> ReadP KeyMask
 parseModifier c =  (string "M-" >> return (modMask c))
                +++ (string "C-" >> return controlMask)
                +++ (string "S-" >> return shiftMask)
-               +++ do char 'M'
+               +++ do _ <- char 'M'
                       n <- satisfy (`elem` ['1'..'5'])
-                      char '-'
+                      _ <- char '-'
                       return $ indexMod (read [n] - 1)
     where indexMod = (!!) [mod1Mask,mod2Mask,mod3Mask,mod4Mask,mod5Mask]
 
@@ -428,11 +431,11 @@ parseRegular = choice [ char s >> return k
 
 -- | Parse a special key name (one enclosed in angle brackets).
 parseSpecial :: ReadP KeySym
-parseSpecial = do char '<'
+parseSpecial = do _   <- char '<'
                   key <- choice [ string name >> return k
                                 | (name,k) <- keyNames
                                 ]
-                  char '>'
+                  _   <- char '>'
                   return key
 
 -- | A list of all special key names and their associated KeySyms.
@@ -648,6 +651,7 @@ multimediaKeys = filter ((/= noSymbol) . snd) . map (id &&& stringToKeysym) $
                  , "XF86MailForward"
                  , "XF86Pictures"
                  , "XF86Music"
+                 , "XF86TouchpadToggle"
                  , "XF86_Switch_VT_1"
                  , "XF86_Switch_VT_2"
                  , "XF86_Switch_VT_3"
