@@ -8,15 +8,18 @@
 -- Stability   :  unstable
 -- Portability :  unportable
 --
+-----------------------------------------------------------------------------
 
 module XMonad.Util.WorkspaceCompare ( WorkspaceCompare, WorkspaceSort
                                     , getWsIndex
                                     , getWsCompare
                                     , getWsCompareByTag
+                                    , getXineramaPhysicalWsCompare
                                     , getXineramaWsCompare
                                     , mkWsSort
                                     , getSortByIndex
                                     , getSortByTag
+                                    , getSortByXineramaPhysicalRule
                                     , getSortByXineramaRule ) where
 
 import XMonad
@@ -25,6 +28,7 @@ import Data.List
 import Data.Monoid
 import Data.Ord
 import Data.Maybe
+import Data.Function
 
 type WorkspaceCompare = WorkspaceId -> WorkspaceId -> Ordering
 type WorkspaceSort = [WindowSpace] -> [WindowSpace]
@@ -41,12 +45,7 @@ getWsIndex = do
 getWsCompare :: X WorkspaceCompare
 getWsCompare = do
     wsIndex <- getWsIndex
-    return $ \a b -> f (wsIndex a) (wsIndex b) `mappend` compare a b
-  where
-    f Nothing Nothing   = EQ
-    f (Just _) Nothing  = LT
-    f Nothing (Just _)  = GT
-    f (Just x) (Just y) = compare x y
+    return $ mconcat [compare `on` wsIndex, compare]
 
 -- | A simple comparison function that orders workspaces
 --   lexicographically by tag.
@@ -57,10 +56,17 @@ getWsCompareByTag = return compare
 --   and screen id. It produces the same ordering as
 --   'XMonad.Hooks.DynamicLog.pprWindowSetXinerama'.
 getXineramaWsCompare :: X WorkspaceCompare
-getXineramaWsCompare = do
+getXineramaWsCompare = getXineramaWsCompare' False
+
+-- | A comparison function like 'getXineramaWsCompare', but uses physical locations for screens.
+getXineramaPhysicalWsCompare :: X WorkspaceCompare
+getXineramaPhysicalWsCompare = getXineramaWsCompare' True
+
+getXineramaWsCompare' :: Bool -> X WorkspaceCompare
+getXineramaWsCompare' phy = do
     w <- gets windowset
     return $ \ a b -> case (isOnScreen a w, isOnScreen b w) of
-        (True, True)   -> comparing (tagToSid (onScreen w)) a b
+        (True, True)   -> cmpPosition phy w a b
         (False, False) -> compare a b
         (True, False)  -> LT
         (False, True)  -> GT
@@ -68,6 +74,10 @@ getXineramaWsCompare = do
     onScreen w =  S.current w : S.visible w
     isOnScreen a w  = a `elem` map (S.tag . S.workspace) (onScreen w)
     tagToSid s x = S.screen $ fromJust $ find ((== x) . S.tag . S.workspace) s
+    cmpPosition False w a b = comparing (tagToSid $ onScreen w) a b
+    cmpPosition True w a b = comparing (rect.(tagToSid $ onScreen w)) a b
+      where rect i = let (Rectangle x y _ _) = screens !! fromIntegral i in (y,x)
+            screens = map (screenRect . S.screenDetail) $ sortBy (comparing S.screen) $ S.current w : S.visible w
 
 -- | Create a workspace sorting function from a workspace comparison
 --   function.
@@ -92,3 +102,7 @@ getSortByTag = mkWsSort getWsCompareByTag
 getSortByXineramaRule :: X WorkspaceSort
 getSortByXineramaRule = mkWsSort getXineramaWsCompare
 
+-- | Like 'getSortByXineramaRule', but uses physical locations for screens.
+getSortByXineramaPhysicalRule :: X WorkspaceSort
+getSortByXineramaPhysicalRule = mkWsSort getXineramaPhysicalWsCompare
+
